// Copyright © 2022 BytePlusRTC All rights reserved.
// SPDX-License-Identifier: MIT

package com.volcengine.reactnative.vertc.vod;

import android.util.Log;

import com.ss.ttvideoengine.TTVideoEngine;
import com.ss.ttvideoengine.VideoEngineCallback;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

// TTVideoEngine 只会保存一个 VideoEngineCallback
// 但是我们需要监听 VideoEngineCallback，会和用户设置的 VideoEngineCallback 冲突
// 因此，我们添加一层代理，同时触发用户的 VideoEngineCallback 和我们内部的 VideoEngineCallback
// 结束后再 reset
public class VodVideoEngineCallbackProxy implements VideoEngineCallback {
  public final String TAG = "VodVideoEngineCallback";
  
  public VideoEngineCallback mOriginalCallback = null;
  public VideoEngineCallback mCallback = null;
  public VideoEngineCallback mProxyCallback = null;
  
  public VodVideoEngineCallbackProxy(TTVideoEngine ttVideoEngine, VideoEngineCallback callback) {
    mCallback = callback;
    mOriginalCallback = findOriginalCallback(ttVideoEngine);
  }
  
  public VideoEngineCallback listen() {
    Class<VideoEngineCallback> clazz = VideoEngineCallback.class;
    Object dynamicObserver = Proxy.newProxyInstance(
      clazz.getClassLoader(), // 使用Observer接口的类加载器
      new Class[]{clazz}, // 指定要代理的接口
      (proxy, method, args) -> {
        if (mOriginalCallback != null) {
          triggerCallback(method, args);
          return invokeOriginal(method, args);
        }
        return triggerCallback(method, args);
      }
    );
    mProxyCallback = (VideoEngineCallback) dynamicObserver;
    return mProxyCallback;
  }

  public void reset(TTVideoEngine ttVideoEngine) {
    if (ttVideoEngine != null) {
      if (findOriginalCallback(ttVideoEngine) == mProxyCallback) {
        ttVideoEngine.setVideoEngineCallback(mOriginalCallback);
      }
    }
  }
  
  private Object invokeOriginal(Method method, Object[] args) {
    try {
      Method originalCallbackMethod = mOriginalCallback.getClass().getMethod(method.getName(), method.getParameterTypes());
      return originalCallbackMethod.invoke(mOriginalCallback, args);
    } catch (Exception ignore) {
      return null;
    }
  }

  private Object triggerCallback(Method method, Object[] args) {
    try {
      Method cbMethod = mCallback.getClass().getMethod(method.getName(), method.getParameterTypes());
      return cbMethod.invoke(mCallback, args);
    } catch (Exception ignore) {
      return null;
    }
  }
  
  private VideoEngineCallback findOriginalCallback(TTVideoEngine ttVideoEngine) {
    if (ttVideoEngine == null) {
      return null;
    }
    try {
      Field mVideoEngineField = TTVideoEngine.class.getDeclaredField("mVideoEngine");
      mVideoEngineField.setAccessible(true);
      Object mVideoEngine = mVideoEngineField.get(ttVideoEngine);
      Field mListenerCompactField = mVideoEngine.getClass().getDeclaredField("mListenerCompact");
      mListenerCompactField.setAccessible(true);
      Object mListenerCompact = mListenerCompactField.get(mVideoEngine);
      Field mEngineCallbackField = mListenerCompact.getClass().getDeclaredField("mEngineCallback");
      mEngineCallbackField.setAccessible(true);
      Object mEngineCallback = mEngineCallbackField.get(mListenerCompact);
      return  (VideoEngineCallback) mEngineCallback;
    } catch (Exception e) {
      Log.w(TAG, "can't find original TTVideoEngine Callback, reason: " + e.getMessage());
      return null;
    }
  }
}
